###############################################################################
###############################################################################
##                                  Delay.tcl
###############################################################################
###############################################################################
## This namespace is used to program delayed downloads into Getleft.
###############################################################################
###############################################################################
## (c) 2001-2008 Andrs Garca Garca. fandom@telefonica.net
## You may distribute the contents of this file under the terms of the GPL v2
###############################################################################
###############################################################################

namespace eval Delay {

###############################################################################
# DelayedCancelDelay
#    Cancel the event that would start or/and the end of a delayed download.
###############################################################################
proc DelayedCancelDelay {} {
    variable delayedDownload
    variable timing
    global   getleftState

    catch {after cancel $delayedDownload(startEvent)}
    catch {after cancel $delayedDownload(endEvent)}

    set getleftState(delaySet)     0
    set getleftState(delayPressed) 0
    set getleftState(delayedDown)  0

    foreach time {startHour endHour startMinute endMinute} {
        set timing($time) 0
    }

    return
}

###############################################################################
# DelayedStartDownload
#    Starts the download at the given time.
###############################################################################
proc DelayedStartDownload {} {
    variable delayedDownload
    variable window
    global getleftState

    if {$delayedDownload(noEnd)==0} {
        set clicksToEnd [expr ($delayedDownload(endTime)-[clock seconds])*1000]
        if {$clicksToEnd<0} {
            set clicksToEnd 0
        }
        set delayedDownload(endEvent) \
            [after $clicksToEnd Delay::DelayedEndDownload]
    }
    if {$delayedDownload(days)==1} {
        DelayedStartTime $delayedDownload(startTime)
    }
    set getleftState(delayedDown) 1
    mainWin::UrlListClearSel
    $mainWin::taskbar(auto) invoke

    return
}

###############################################################################
# DelayedEndDownload
#    Ends the download in the given time.
###############################################################################
proc DelayedEndDownload {} {
    variable delayedDownload
    global   getleftOptions

    switch $delayedDownload(how) {
        0 {
            ::Ventana::CancelDownloading .
        }
        1 {
            set getleftOptions(stopFile) 1
            $mainWin::taskbar(auto) invoke
        }
        2 {
            set getleftOptions(stopPage) 1
            $mainWin::taskbar(auto) invoke
        }
    }
    return
}

###############################################################################
# DelayedStartTime
#    Calculates the time the next download will begin.
#
# Parameter
#    The time in seconds from 01-01-1970 when the download will start.
###############################################################################
proc DelayedStartTime {startTime} {
    variable delayedDownload

    set now [clock seconds]
    if {$delayedDownload(days)==1} {
        set hour   [clock format $startTime -format "%H"]
        set minute [clock format $startTime -format "%M"]
        set startTime 9999999999
        foreach day [list monday tuesday  wednesday thursday \
                          friday saturday sunday] {
            if {$delayedDownload($day)==1} {
                set startTmp [clock scan "$day $hour:$minute"]
                if {$startTmp<$now} {
                    set startTmp [expr $startTmp+7*24*3600]
                }
                if {($startTmp<=$startTime)&&($startTmp>$now)} {
                    set startTime $startTmp
                }
            }
        }
    }
    set delayedDownload(startTime) $startTime
    DelayedSetStartTime

    return
}

###############################################################################
# DelayedSetStartTime
#    Given the time when to start, this procedure will create the event to
#    start the download at that time.
###############################################################################
proc DelayedSetStartTime {} {
    variable delayedDownload

    set clicksToStart [expr ($delayedDownload(startTime)-[clock seconds])*1000]
    # This may happen if we closed Getleft and we are running it again.
    if {$clicksToStart<0} {
        if {$delayedDownload(days)==1} {
            DelayedStartTime $delayedDownload(startTime)
        }
        return
    }
    set delayedDownload(startEvent)  \
            [after $clicksToStart Delay::DelayedStartDownload]

    return
}

###############################################################################
# DelayedEndTime
#    Calculates the time the next download will stop.
#
# Parametes
#    The hour and the minute of the time and the start time.
#
# Returns:
#    The time in seconds from 01-01-1970
###############################################################################
proc DelayedEndTime {hour minute startTime} {
    variable delayedDownload

    set endTime [clock scan "today $hour:$minute"]
    while {$startTime>=$endTime} {
        set endTime [expr $endTime + 3600*24]
    }
    return $endTime
}

###############################################################################
# DelayedCheckTime
#    Checks whether the given time makes sense or not.
#
# Parametes
#    The hour and the minute of the time to check.
#
# Returns:
#    '0' makes sense, '1' doesn't.
###############################################################################
proc DelayedCheckTime {hour minute} {

    if {(![string is integer $hour])||(![string is integer $minute])} {
        return 1
    }
    if {($hour<0)||($hour>23)||($minute<0)||($minute>59)} {
        return 1
    }

    return 0
}

###############################################################################
# DelayedActivateEndTime
#    Activates or deactivates the spinboxes with the time to finish downloading
###############################################################################
proc DelayedActivateEndTime {} {
    variable delayedDownloadTemp
    variable window

    if {$delayedDownloadTemp(noEnd)==1} {
        $window(endHour)   configure -state disable
        $window(endMinute) configure -state disable
    } else {
        $window(endHour)   configure -state normal
        $window(endMinute) configure -state normal
    }

    return
}

###############################################################################
# DelayedActivateDays
#    Activates or deactivates the 'days' buttons.
###############################################################################
proc DelayedActivateDays {} {
    variable delayedDownloadTemp
    variable dayButton

    if {$delayedDownloadTemp(days)==0} {
        $dayButton(monday)    configure -state disable
        $dayButton(tuesday)   configure -state disable
        $dayButton(wednesday) configure -state disable
        $dayButton(thursday)  configure -state disable
        $dayButton(friday)    configure -state disable
        $dayButton(saturday)  configure -state disable
        $dayButton(sunday)    configure -state disable
    } else {
        if {[$dayButton(monday) cget -state]!="normal"} {
            $dayButton(monday)    configure -state normal
            $dayButton(tuesday)   configure -state normal
            $dayButton(wednesday) configure -state normal
            $dayButton(thursday)  configure -state normal
            $dayButton(friday)    configure -state normal
            $dayButton(saturday)  configure -state normal
            $dayButton(sunday)    configure -state normal
        }
    }
    return
}

###############################################################################
# DelayedActivateStop
#    Activates or deactivates the 'days' buttons.
###############################################################################
proc DelayedActivateStop {} {
    variable delayedDownloadTemp
    variable stopButton

    if {$delayedDownloadTemp(noEnd)==0} {
        if {[$stopButton(inme) cget -state]!="normal"} {
            $stopButton(inme) configure -state normal
            $stopButton(file) configure -state normal
            $stopButton(page) configure -state normal
        }
    } else {
        $stopButton(inme) configure -state disable
        $stopButton(file) configure -state disable
        $stopButton(page) configure -state disable
    }
    return
}

###############################################################################
# DelayedDownloadWindow
#    Creates the dialog to define when to start a download.
###############################################################################
proc DelayedDownloadWindow {} {
    global labelDialogs labelTitles labelFrames getleftState getleftOptions
    global indexButtons indexDialogs
    variable delayedDownloadTemp
    variable dayButton
    variable stopButton
    variable window
    variable timing

    if {[winfo exists .delayedDown]} {
        wm deiconify .delayedDown
        return
    }

    set win [toplevel .delayedDown]
    wm title $win $labelTitles(delayedDown)
    wm resizable $win 0 0
    keynav::enableMnemonics $win

    set coord(x) [winfo rootx .]
    set coord(y) [winfo rooty .]
    wm geometry $win +[expr {$coord(x)+100}]+[expr {$coord(y)+15}]

    set extFrame [ttk::frame $win.extFrame -class ExternalFrame]
    set intFrame [ttk::frame $extFrame.intFrame]

    set timeFrame [ttk::labelframe $intFrame.time  -labelanchor nw              \
            -labelwidget [ttk::label $intFrame.timelb -textvariable labelFrames(time)]]
    raise $intFrame.timelb
    set startLabel  [ttk::label $timeFrame.startLabel -textvariable labelDialogs(begin)]
    set startHour   [spinbox $timeFrame.startHour                               \
            -textvariable Delay::timing(startHour)                              \
            -from 0 -to 23 -width 3 -justify right                              \
            -background $getleftOptions(bg) -foreground $getleftOptions(fg)]
    set startMinute [spinbox $timeFrame.startMinute                             \
            -textvariable Delay::timing(startMinute)                            \
            -from 0 -to 59 -width 3 -increment 5 -justify right                 \
            -background $getleftOptions(bg) -foreground $getleftOptions(fg)]
    set endLabel    [ttk::label $timeFrame.endLabel -textvariable labelDialogs(end)]
    set endHour     [spinbox $timeFrame.endHour                                 \
            -textvariable Delay::timing(endHour)                                \
            -from 0 -to 23 -width 3 -justify right                              \
            -background $getleftOptions(bg) -foreground $getleftOptions(fg)     \
            -disabledforeground $getleftOptions(disFg)                          \
            -disabledbackground $getleftOptions(disBg)]
    set endMinute   [spinbox $timeFrame.endMinute                               \
            -textvariable Delay::timing(endMinute)                              \
            -from 0 -to 59 -width 3 -increment 5 -justify right                 \
            -bg $getleftOptions(bg) -fg $getleftOptions(fg)                     \
            -disabledbackground $getleftOptions(disBg)                          \
            -disabledforeground $getleftOptions(disFg)                          \
            ]
    set noEnd [ttk::checkbutton $timeFrame.noEnd                                \
            -textvariable labelDialogs(noLimit) -underline $indexDialogs(noLimit)\
            -variable Delay::delayedDownloadTemp(noEnd) -command "
                Delay::DelayedActivateEndTime
                Delay::DelayedActivateStop
            "]
    if {![info exists delayedDownloadTemp(noEnd)]} {
        set delayedDownloadTemp(noEnd) 1
    }
    if {![catch {clock format $delayedDownloadTemp(startTime)}]} {
        set timing(startHour)                                                  \
                [clock format $delayedDownloadTemp(startTime) -format "%H"]
        set timing(startMinute)                                                \
                [clock format $delayedDownloadTemp(startTime) -format "%M"]
    }
    if {![catch {clock format $delayedDownloadTemp(endTime)}]} {
        set timing(endHour)                                                    \
                [clock format $delayedDownloadTemp(endTime) -format "%H"]
        set timing(endMinute)                                                  \
                [clock format $delayedDownloadTemp(endTime) -format "%M"]
    }

    set window(win)         $win
    set window(startHour)   $startHour
    set window(startMinute) $startMinute
    set window(endHour)     $endHour
    set window(endMinute)   $endMinute

    DelayedActivateEndTime

    set howFrame  [ttk::labelframe $intFrame.how -labelanchor nw               \
            -labelwidget [ttk::label $intFrame.howlb -textvariable labelFrames(stop)]]
    raise $intFrame.howlb
    set stopButton(inme) [ttk::radiobutton $howFrame.inme                      \
            -textvariable labelDialogs(inme)                                   \
            -variable Delay::delayedDownloadTemp(how) -value 0                 \
            -underline $indexDialogs(inme)]
    set stopButton(file) [ttk::radiobutton $howFrame.file                      \
            -textvariable labelDialogs(afterFile)                              \
            -variable Delay::delayedDownloadTemp(how) -value 1                 \
            -underline $indexDialogs(afterFile)]
    set stopButton(page) [ttk::radiobutton $howFrame.page                      \
            -textvariable labelDialogs(afterPage)                              \
            -variable Delay::delayedDownloadTemp(how) -value 2                 \
            -underline $indexDialogs(afterPage)]
    if {![info exists delayedDownloadTemp(how)]} {
        set delayedDownloadTemp(how) 1
    }
    DelayedActivateStop

    set daysFrame [ttk::labelframe $intFrame.days -labelanchor nw              \
            -labelwidget [ttk::label $intFrame.dayslb -textvariable labelFrames(days)]]
    raise $intFrame.dayslb
    set today [ttk::radiobutton $daysFrame.today                               \
            -textvariable labelDialogs(onlyToday)                              \
            -variable Delay::delayedDownloadTemp(days) -value 0                \
            -underline $indexDialogs(onlyToday)                                \
            -command Delay::DelayedActivateDays]
    set on    [ttk::radiobutton $daysFrame.on                                  \
            -textvariable labelDialogs(on) -underline $indexDialogs(on)        \
            -variable Delay::delayedDownloadTemp(days) -value 1                \
            -command Delay::DelayedActivateDays]
    if {![info exists delayedDownloadTemp(days)]} {
        set delayedDownloadTemp(days) 0
    }

    set dayButton(monday)    [ttk::checkbutton $daysFrame.monday               \
            -textvariable labelDialogs(monday)                                 \
            -variable Delay::delayedDownloadTemp(monday)                       \
            -underline $indexDialogs(monday)]
    set dayButton(tuesday)   [ttk::checkbutton $daysFrame.tuesday              \
            -textvariable labelDialogs(tuesday)                                \
            -variable Delay::delayedDownloadTemp(tuesday)                      \
            -underline $indexDialogs(tuesday)]
    set dayButton(wednesday) [ttk::checkbutton $daysFrame.wednesday            \
            -textvariable labelDialogs(wednesday)                              \
            -variable Delay::delayedDownloadTemp(wednesday)                    \
            -underline $indexDialogs(wednesday)]
    set dayButton(thursday)  [ttk::checkbutton $daysFrame.thursday             \
            -textvariable labelDialogs(thursday)                               \
            -variable Delay::delayedDownloadTemp(thursday)                     \
            -underline $indexDialogs(thursday)]
    set dayButton(friday)    [ttk::checkbutton $daysFrame.friday               \
            -textvariable labelDialogs(friday)                                 \
            -variable Delay::delayedDownloadTemp(friday)                       \
            -underline $indexDialogs(friday)]
    set dayButton(saturday)  [ttk::checkbutton $daysFrame.saturday             \
            -textvariable labelDialogs(saturday)                               \
            -variable Delay::delayedDownloadTemp(saturday)                     \
            -underline $indexDialogs(saturday)]
    set dayButton(sunday)    [ttk::checkbutton $daysFrame.sunday               \
            -textvariable labelDialogs(sunday)                                 \
            -variable Delay::delayedDownloadTemp(sunday)                       \
            -underline $indexDialogs(sunday)]
    DelayedActivateDays

    foreach day {monday tuesday wednesday thursday friday saturday sunday} {
        set delayedDownloadTemp($day) 0
    }

    set buttonFrame [ttk::frame $extFrame.buttonFramec -class BottonFrame]
    set accept [ttk::button $buttonFrame.accept                  \
        -textvariable labelButtons(ok)                           \
        -width 8 -command "Delay::DelayedDownloadControl 1"      \
        -underline $indexButtons(ok)]
    set cancel [ttk::button $buttonFrame.cancel                  \
        -textvariable labelButtons(cancel)                       \
        -width 8 -command "Delay::DelayedDownloadControl 0"      \
        -underline $indexButtons(cancel)]
    set clear  [ttk::button $buttonFrame.clear                   \
         -textvariable labelButtons(clear)                       \
        -width 8 -command "Delay::DelayedDownloadControl 2"      \
        -underline $indexButtons(clear)]

    pack $extFrame
    pack $intFrame -padx 5

    if {$::tcl_version>=8.5} {
        grid anchor $timeFrame  c
        grid anchor $howFrame   c
        grid anchor $daysFrame  c
    }

    grid $timeFrame $howFrame     -sticky news -ipadx 7 -ipady 7 -padx 5
    grid $daysFrame -columnspan 2 -sticky news -ipadx 7 -ipady 7 -padx 5

    grid $startLabel $startHour $startMinute -padx 5 -pady 3 -sticky w
    grid $endLabel   $endHour   $endMinute   -padx 5 -pady 3 -sticky w
    grid $noEnd -columnspan 3

    grid $stopButton(inme) -sticky w
    grid $stopButton(file) -sticky w
    grid $stopButton(page) -sticky w

    grid $today $on  -sticky w
    grid $dayButton(monday)    $dayButton(tuesday)                     -sticky w
    grid $dayButton(wednesday) $dayButton(thursday) $dayButton(friday) -sticky w
    grid $dayButton(saturday)  -row 3 -column 1 -sticky w
    grid $dayButton(sunday)    -row 3 -column 2 -sticky w

    pack $buttonFrame           -padx 7 -pady 5   -side right
    if {$getleftState(os) ne "mac"} {
        pack $cancel $accept $clear -padx 2       -side right
    } else {
        pack $accept $cancel $clear -padx 2       -side right
    }

    bind $win <Escape> "$cancel invoke"
    wm protocol $win   WM_DELETE_WINDOW "$cancel invoke"

    if {$getleftState(delaySet)==0} {
        $clear configure -state disable
    } else {
        $clear configure -state normal
    }

    return
}

###############################################################################
# DelayedDownloadControl
#    Checks that the data entered in the dialog is consistent.
#
# Parameter
#    '1' if the user accept the data, '0' if the dialog was cancelled.
###############################################################################
proc DelayedDownloadControl {done} {
    global labelTitles labelMessages getleftState
    variable window
    variable timing
    variable delayedDownload
    variable delayedDownloadTemp

    if {$done==1} {
        set startHour   $timing(startHour)
        set startMinute $timing(startMinute)
        if {[DelayedCheckTime $startHour $startMinute]==1} {
            Dialogos::Dialogo $window(win).error -title $labelTitles(error)     \
                    -message $labelMessages(wrongStart) -type ok
            return
        }
        if {$delayedDownloadTemp(noEnd)==0} {
            set endHour   $timing(endHour)
            set endMinute $timing(endMinute)

            if {[DelayedCheckTime $endHour $endMinute]==1} {
                Dialogos::Dialogo $window(win).error -title $labelTitles(error) \
                        -message $labelMessages(wrongStart) -type ok
                return
            }
        }
        if {$delayedDownloadTemp(days)==1} {
            if {($delayedDownloadTemp(monday)==0)      &&($delayedDownloadTemp(tuesday)==0) \
                 &&($delayedDownloadTemp(wednesday)==0)&&($delayedDownloadTemp(thursday)==0)\
                 &&($delayedDownloadTemp(friday)==0)   &&($delayedDownloadTemp(saturday)==0)\
                 &&($delayedDownloadTemp(sunday)==0)} {
                Dialogos::Dialogo $window(win).error -title $labelTitles(error)             \
                        -message $labelMessages(whichDays) -type ok
                return
            }
        }
    }

    destroy $window(win)

    if {$done==0} {
        set getleftState(delayPressed) $getleftState(delaySet)
        return
    }

    if {$done==2} {
        DelayedCancelDelay
        return
    }
    DelayedCancelDelay
    array set delayedDownload [array get delayedDownloadTemp]
    set startTime [clock scan "today $startHour:$startMinute"]
    set now [clock seconds]
    if {$startTime<$now} {
        set startTime [expr $startTime+24*3600]
    }
    DelayedStartTime $startTime
    if {$delayedDownload(noEnd)==0} {
        set delayedDownload(endTime)         \
                [DelayedEndTime $endHour $endMinute $delayedDownload(startTime)]
    }

    set getleftState(delaySet)     1
    set getleftState(delayPressed) 1

    return
}

###############################################################################
# DelayedDownload
#    Takes care of all the trivia of defining a delayed download.
###############################################################################
proc DelayedDownload {} {
    variable delayedDownloadTemp
    variable delayedDownload

    catch {unset delayedDownloadTemp}
    array set delayedDownloadTemp [array get delayedDownload]

    DelayedDownloadWindow

    return
}

}
